/*
 *  Copyright (C) 2004 Karlheinz Klingbeil (lunqual)
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package de.lunqual.rzpro;
// eigene Pakete
import java.awt.Cursor;
import java.awt.Window;

import de.lunqual.rzpro.ausgabe.AusgabeFactory;
import de.lunqual.rzpro.ausgabe.ReportFactory;
import de.lunqual.rzpro.database.DBFactory;
import de.lunqual.rzpro.database.DBImport;
import de.lunqual.rzpro.fenster.dialoge.DialogControl;
import de.lunqual.rzpro.fensterfactory.FensterFactory;
import de.lunqual.rzpro.fonts.FontFactory;
import de.lunqual.rzpro.hilfefactory.HilfeFactory;
import de.lunqual.rzpro.items.message.MessageItem;
import de.lunqual.rzpro.localefactory.LocaleFactory;
import de.lunqual.rzpro.log.LogFactory;
import de.lunqual.rzpro.options.OptionFactory;
import de.lunqual.rzpro.tafel.TafelFactory;

// java-pakete

/**
 * rzpro ist die Hauptklasse, von der aus alles andere gestartet wird
 * ein Verweis auf rzpro wird jeder anderen Klasse mitgegeben.
 * die "Controls" übernehmen spezielle Aufgaben und sind über
 * diese Klasse aufrufbar...
 * @author  lunqual
 */

public class Import  extends RzPro{

    //Const
    // Application allgemein
    public static final String      	APP_NAME            = "Rezeptbuch Pro";
    public static final int         	APP_VERSION         = 0;
    public static final int         	APP_MAJOR           = 1;
    public static final int         	APP_MINOR           = 1;

    // maximale Länger der Strings in der Datenbank
    public static final int         	STRING_SIZE         		= 255;
    //nach xxx millisekunden läuft der Timer für listen ab
    public static final int         	LIST_TIMEOUT        		= 250;
    // nach xxx millisekunden läuft der Timer im Zutatendialog ab
    public static final int			ZUTATEN_TIMEOUT		= 125;

    // soll Garbage Collection bei jedem "Sperren" erfolgen ?
    public static final String      DEFAULT_GC_HANDLING = "yes";

    public static final double 	DOUBLE_NULL = 0.00001;

    // Voreinstellungen
    public static final int         OPTION_KALKULATION_DEFAULT_CURRENCY = 1;
    public static final int         CURRENCY_DEFAULT_SCALE = 4;
    public static final int         OPTION_REZEPTUR_DEFAULT_EINHEIT = 1;

    public static final String     DATEFORMAT_NORMAL   		= "dd.MM.yyyy H:mm:ss";
    public static final String     DATEFORMAT          			= "EEEEE, d. MMMMM yyyy ";

    // fundamentale Rezepturtypen
    public static final int         TYP_ID_REZEPT       			= 1;
    public static final int         TYP_ID_NOTIZ        				= 2;
    public static final int		  TYP_ID_VERWEIS    				= 3;
    //Zahlenformate


    // fundamentale Zeilentypen
    public static final String      TYP_COMMENT         			= "0";
    public static final String      TYP_CALC            				= "1";
    // einheiten
    public static final int			EINHEIT_COMMENT			= 0;
    public static final int         	EINHEIT_LITER       			= 1;
    public static final int         	EINHEIT_KG          				= 2;
    public static final int         	EINHEIT_LA          				= 3;
    public static final int			EINHEIT_STAERKE				= 4;
    public static final int			EINHEIT_EXTRAKT				= 5;
    public static final int 			EINHEIT_FAKTOR				= 6;

    LogFactory         	log;
    OptionFactory      	options;
    FontFactory         	fonts;
    LocaleFactory      	locale;
    FensterFactory    	fenster;
    DBFactory           	database;
    DialogControl       	dialog;
    AusgabeFactory  	ausgabe;
    ReportFactory      	report;
    HilfeFactory        	hilfe;
    TafelFactory        	tafel;
    DBImport				db_import;

    /**  gibt an, ob das Programm
     * gerade gestartet wurde oder
     * ob es schon läuft
     * wird zum Sperren/Entsperren
     * gebraucht
     */
    boolean             	startup = true;

    /**
     * Creates a new instance of rzpro
     */
    public Import(String[] args){
    	super();
    }

    public void run() {
    	openApp();
    }
    /** das Programm starten.. alle nötigen Klassen instanzieren  */
    @Override
	public void openApp(){
        log         	= new LogFactory(this);
        options     	= new OptionFactory(this);
        locale      	= new LocaleFactory(this); // Sprache    ;
        database   = new DBFactory(this);
        fonts       	= new FontFactory(this);
        fenster    	= new FensterFactory(this);
        dialog      	= new DialogControl(this);
        ausgabe    = new AusgabeFactory(this);
        report      	= new ReportFactory(this);
        hilfe       	= new HilfeFactory(this);
        tafel       	= new TafelFactory(this);

        //fenster.getFensterListe().createWindow(FensterControl.FENSTER_LOGIN,true,null);

        if(startup){
            startup=false;
            System.out.println("import");
            if(database.dbLogin("com.mysql.jdbc.Driver", "jdbc:mysql://localhost:3306/rzpro", "root", "")) {
                db_import = new DBImport(this,database,database.getConnection());

            	db_import.dbClose();
            }
        	closeApp();
        }
    }

    /**
     * putMsg nimmt Nachrichten anderer Fenster entgegen...
     * @param messageItem
     *
     * */

    public void putMessage(final MessageItem msg){

    }

    /** getVersionString gibt den Namen und die Version zurück  */
    @Override
	public String getVersionString(){
        return APP_NAME + " (" + APP_VERSION + "." + APP_MAJOR + "." + APP_MINOR + ")";
    }


    /** aufräumen und Programm beenden */
    @Override
	public void closeApp(){
        database.dbClose();
        fenster.disposeAllWindows();
        options.cleanUp();
        log=null;
        options = null;
        locale = null;
        fenster = null;
        fonts = null;
        database = null;
        dialog = null;
        ausgabe = null;
        report = null;
        hilfe = null;
        tafel = null;
        System.exit(0);
    }

    @Override
	public boolean isZero(final double value) {
        return Math.abs(value) < DOUBLE_NULL;
    }

    @Override
	public void setWaitCursor(final Window w){
        if(w != null) {
			w.setCursor(Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR));
		}
    }

    @Override
	public void setFreeCursor(final Window w) {
      if(w != null) {
		w.setCursor(Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR));
	}
    }
    @Override
	public LogFactory       getLogFactory(){return log;}
    @Override
	public OptionFactory    getOptionFactory(){return options;}
    @Override
	public LocaleFactory    getLocale(){return locale;}
    @Override
	public FensterFactory   getFensterFactory(){return fenster;}
    @Override
	public FontFactory      getFontFactory(){return fonts;}
    @Override
	public DBFactory        getDatabase(){return database;}
    @Override
	public DialogControl    getDialogFactory(){return dialog;}
    @Override
	public AusgabeFactory   getAusgabeFactory(){return ausgabe;}
    @Override
	public ReportFactory    getReportFactory(){return report;}
    @Override
	public HilfeFactory     getHilfeFactory(){return hilfe;}
    @Override
	public TafelFactory     getTafelFactory(){return tafel;}

    /** @param args the command line arguments   */
    public static void main(final String[] args) {

    try {

    } catch (final Exception e) { System.out.println("RzPro Startup Error :" + e.getLocalizedMessage());}
        final Import r = new Import(args);
        r.run();
    }

}
